import { Component } from '@angular/core';
import { UserAnnuary } from '../models/user.model';
import { AuthService } from '../services/auth.service';
import { SearchService } from '../structure-list/services/search.service';
import { SearchQuery, SearchResults } from './models/searchQuery.model';

@Component({
  selector: 'app-annuaire',
  templateUrl: './annuaire.component.html',
  styleUrls: ['./annuaire.component.scss'],
})
export class AnnuaireComponent {
  constructor(private searchService: SearchService, private authService: AuthService) {}
  public userList: UserAnnuary[] = [];
  public searchedValue: string;
  public totalUserResult: number;
  public resetFilters: number = 0;
  public nextPage: number = 1;
  public filterActive: boolean = false;

  ngOnInit(): void {
    this.getUsers({ queryParam: '', page: 1, jobFilters: [], employerFilters: [] });
  }

  public getUsers(params: SearchQuery): void {
    if (this.userIsLoggedIn()) {
      this.searchService.annuaireSearchQuery = params;
      if (params.employerFilters?.length || params.jobFilters?.length) {
        this.filterActive = true;
      } else {
        this.filterActive = false;
      }
      this.searchService
        .searchUserRegistry(params.queryParam, params.page, params.jobFilters, params.employerFilters)
        .subscribe((res: SearchResults) => {
          this.userList = res.docs;
          this.totalUserResult = res.count;
        });
    } else {
      this.searchService.getUserRegistryCount().subscribe((nb: number) => {
        this.totalUserResult = nb;
      });
    }
  }
  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }
  public shouldResetFilters(): void {
    this.resetFilters++;
  }
  public showMore(): void {
    this.nextPage++;
  }
}
