import { Component, Input, Output, EventEmitter, OnInit, OnChanges, SimpleChanges } from '@angular/core';
import { Router } from '@angular/router';
import { UserAnnuary } from '../../models/user.model';
import { ButtonType } from '../../shared/components/button/buttonType.enum';

@Component({
  selector: 'app-result-list',
  templateUrl: './result-list.component.html',
  styleUrls: ['./result-list.component.scss'],
})
export class ResultListComponent implements OnInit, OnChanges {
  constructor(private router: Router) {}
  @Input() userList: UserAnnuary[];
  @Input() totalUserResult: number;
  @Input() isLogged: boolean;
  @Input() filterActive: boolean;
  @Output() resetEvent = new EventEmitter<any>();
  @Output() showMoreEvent = new EventEmitter<any>();
  public maxPerPage: number = 20;
  public buttonTypeEnum = ButtonType;
  public showPagination: boolean = false;

  ngOnInit(): void {
    if (this.totalUserResult > this.maxPerPage) {
      this.showPagination = true;
    }
  }
  ngOnChanges(changes: SimpleChanges) {
    if (changes.totalUserResult && changes.totalUserResult.currentValue > this.maxPerPage) {
      this.showPagination = true;
    }
    if (this.userList.length >= this.totalUserResult) this.showPagination = false;
  }
  public goToUser(userId: string): void {
    this.router.navigateByUrl(`/profile/${userId}`);
  }
  public goLogin(): void {
    this.router.navigate(['/login'], { queryParams: { returnUrl: '/annuaire' } });
  }
  public goRegister(): void {
    this.router.navigateByUrl('/form/account');
  }
  public resetFilters(): void {
    this.resetEvent.emit();
  }
  public showMore(): void {
    if (this.userList.length < this.totalUserResult) this.showMoreEvent.emit();
  }
}
