import { Component, EventEmitter, Input, Output } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { User } from '../../../models/user.model';
import { Filter } from '../../../structure-list/models/filter.model';
import { OrientationUtils } from '../../../utils/orientationUtils';
import { FiltersSteps } from '../enums/filtersSteps.enum';
import { GenericOrientationSteps } from '../enums/genericOrientationSteps.enum';
import { RecapsType } from '../enums/recapsType.enum';
import { StructuresListSteps } from '../enums/structuresListSteps.enum';
import { FiltersForm } from '../interfaces/filtersForm.interface';
import { StructureOrientationForm } from '../interfaces/structureOrientationForm.interface';

@Component({
  selector: 'app-equipment-buy',
  templateUrl: './equipment-buy.component.html',
})
export class EquipmentBuyComponent {
  @Input() currentStep: FiltersSteps | StructuresListSteps;
  @Input() currentType: GenericOrientationSteps;
  @Input() form: FormGroup<FiltersForm>;
  @Input() orientationForm: FormGroup<StructureOrientationForm>;
  @Input() filters: Filter[] = [];
  @Input() profile: User;
  @Output() validatePage = new EventEmitter<any>();

  public orientationUtils = new OrientationUtils();
  public pagesValidation: any[] = [];

  // Enums
  public FiltersSteps = FiltersSteps;
  public recapsType = RecapsType;
  public GenericOrientationSteps = GenericOrientationSteps;

  public checkValidation(): void {
    switch (this.currentType) {
      case GenericOrientationSteps.common:
        this.orientationUtils.setValidationsEquipmentForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as FiltersSteps
        );
        break;
      case GenericOrientationSteps.structureList:
        this.orientationUtils.setValidationsStructuresForm(
          this.pagesValidation,
          this.orientationForm,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as StructuresListSteps
        );
        break;
      default:
        throw new Error('Not implemented tunnel type in OnlineDemarchFormComponent');
    }
  }
}
