import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { DateTime } from 'luxon';

@Component({
  selector: 'app-mediation-hours-selection',
  templateUrl: './mediation-hours-selection.component.html',
  styleUrls: ['./mediation-hours-selection.component.scss'],
})
export class MediationHoursSelectionComponent implements OnInit {
  @Input() form: UntypedFormGroup;
  @Output() checkValidation = new EventEmitter<any>();
  public selected: string;
  public timeSlots = this.setDateSlot();

  ngOnInit(): void {
    this.checkValidation.emit();
    if (this.form.get('dateSlot').value) {
      this.selected = this.form.get('dateSlot').value.day;
    }
    const today = DateTime.local().weekday;
    if (today !== 7) {
      const temp = this.timeSlots.splice(today, this.timeSlots.length);
      this.timeSlots = temp.concat(this.timeSlots);
    }
  }

  private setDateSlot(): any {
    let date = new Date();
    let dateSlot = [];
    for (let i = 0; i < 7; i++) {
      date.setDate(date.getDate() + 1);
      dateSlot[date.getDay()] = date.toLocaleDateString('fr-FR', {
        month: 'numeric',
        day: 'numeric',
      });
    }

    return [
      { day: `Lundi ${dateSlot[1]}`, hours: '15h00 - 17h00' },
      { day: `Mardi ${dateSlot[2]}`, hours: '15h00 - 17h00' },
      { day: `Mercredi ${dateSlot[3]}`, hours: '15h00 - 17h00' },
      { day: `Jeudi ${dateSlot[4]}`, hours: '15h00 - 19h00' },
      { day: `Vendredi ${dateSlot[5]}`, hours: '15h00 - 17h00' },
      { day: `Samedi ${dateSlot[6]}`, hours: '10h00 - 12h00' },
    ];
  }
  public selectHour(slot: any): void {
    this.selected = slot.day;
    this.form.get('dateSlot').patchValue(slot);
    this.checkValidation.emit();
  }
  public isSelected(slot: any): boolean {
    return this.selected === slot.day;
  }
}
