import { Component, EventEmitter, Input, Output } from '@angular/core';
import { UntypedFormArray, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { User } from '../../../models/user.model';
import { Category } from '../../../structure-list/models/category.model';
import { Filter } from '../../../structure-list/models/filter.model';
import { OrientationUtils } from '../../../utils/orientationUtils';
import { HotlineMediationSteps } from '../enums/hotlineMediationSteps.enum';
import { MeetingRDVSSteps } from '../enums/MeetingRDVSSteps.enum';
import { OnlineDemarche } from '../enums/onlineDemarche.enum';
import { OnlineDemarchesCommonSteps } from '../enums/onlineDemarchesCommonSteps.enum';
import { RecapsType } from '../enums/recapsType.enum';
import { StructuresListSteps } from '../enums/structuresListSteps.enum';
import { OnlineMediationSteps } from '../types/onlineDemarcheStep.type';

@Component({
  selector: 'app-onlineDemarch-form',
  templateUrl: './onlineDemarch-form.component.html',
})
export class OnlineDemarchFormComponent {
  @Input() currentStep: OnlineMediationSteps | StructuresListSteps | MeetingRDVSSteps;
  @Input() currentType: OnlineDemarche;
  @Input() form: UntypedFormGroup;
  @Input() filters: Filter[] = [];
  @Input() profile: User;
  @Input() categories: { [key: string]: Category | [] };
  @Output() validatePage = new EventEmitter<any>();
  @Output() setResetOrientation = new EventEmitter<any>();

  public orientationUtils = new OrientationUtils();
  public pagesValidation: any[] = [];

  // Enums
  public HotlineMediationStepsEnum = HotlineMediationSteps;
  public onlineDemarchesCommonStepsEnum = OnlineDemarchesCommonSteps;
  public structuresListStepsEnum = StructuresListSteps;
  public onlineDemarcheTypeEnum = OnlineDemarche;
  public onlineDemarchesMeetingSteps = MeetingRDVSSteps;
  public recapsType = RecapsType;

  public checkValidation(): void {
    switch (this.currentType) {
      case OnlineDemarche.onlineMediation:
        this.orientationUtils.setValidationsOnlineMediationForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as HotlineMediationSteps
        );
        break;
      case OnlineDemarche.structureList:
        this.orientationUtils.setValidationsStructuresForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as StructuresListSteps
        );
        break;
      case OnlineDemarche.common:
        this.orientationUtils.setValidationsOnlineDemarchesForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as OnlineDemarchesCommonSteps
        );
        break;
      case OnlineDemarche.meetings:
        this.orientationUtils.setValidationsOnlineMeetingForm(
          this.pagesValidation,
          this.form,
          (isValid) => this.validatePage.emit(isValid),
          this.currentStep as MeetingRDVSSteps
        );
        break;
      default:
        throw new Error('Not implemented tunnel type in OnlineDemarchFormComponent');
    }
  }

  public updatePublicChoice({ formControlName, choice }: { formControlName: string; choice: string }): void {
    const event = !this.isInArray({ formControlName, term: choice });
    this.onCheckChange({ event, formControlName, value: choice });
  }

  public onCheckChange({
    event,
    formControlName,
    value,
  }: {
    event: boolean;
    formControlName: string;
    value: string;
  }): void {
    const formArray: UntypedFormArray = this.form.get(formControlName) as UntypedFormArray;
    if (event) {
      // Add a new control in the arrayForm
      formArray.push(new UntypedFormControl(value));
    } else {
      // Remove uncheck control in the arrayForm
      const index = formArray.value.findIndex((element) => element === value);
      formArray.removeAt(index);
    }
    this.checkValidation();
  }
  // Check if a FormControl value is in FormArray
  public isInArray({ formControlName, term }: { formControlName: string; term: string }): boolean {
    if (this.form.get(formControlName) && this.form.get(formControlName).value) {
      return this.form.get(formControlName).value.includes(term);
    }
    return false;
  }
  public onRadioChange(nameAndEvent: { name: string; value: boolean }): void {
    const { name, value } = nameAndEvent;
    this.form.get(name).setValue(value);
    this.checkValidation();
  }
  public showResetOrientation(): void {
    this.setResetOrientation.emit();
  }
}
