import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { StructureWithOwners } from '../../models/structureWithOwners.model';
import { TempUser } from '../../models/temp-user.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { CustomRegExp } from '../../utils/CustomRegExp';

@Component({
  selector: 'app-structure-add-member-modal',
  templateUrl: './structure-add-member-modal.component.html',
  styleUrls: ['./structure-add-member-modal.component.scss'],
})
export class StructureAddMemberModalComponent implements OnInit {
  @Input() public structure: StructureWithOwners;
  @Output() closed = new EventEmitter();
  public buttonTypeEnum = ButtonType;
  public formAddAccount: FormGroup;
  public ownerAlreadyLinked = false;

  constructor(
    private formBuilder: FormBuilder,
    private structureService: StructureService,
    private notificationService: NotificationService
  ) {}

  ngOnInit(): void {
    this.formAddAccount = this.formBuilder.group({
      email: ['', [Validators.required, Validators.pattern(CustomRegExp.EMAIL)]],
    });
  }

  public closeModal(value: boolean): void {
    this.closed.emit(value);
  }

  // getter for form fields
  get fAddAccount(): { [key: string]: AbstractControl } {
    return this.formAddAccount.controls;
  }

  public addOwner(): void {
    // stop here if form is invalid
    if (this.formAddAccount.invalid) {
      return;
    }
    const user = new TempUser();
    user.email = this.fAddAccount.email.value;
    this.structureService.addOwnerToStructure(user, this.structure.structure._id).subscribe(
      (res) => {
        if ((res as TempUser).email) {
          this.notificationService.showSuccess("La demande d'ajout a bien été effectuée");
        } else {
          this.notificationService.showSuccess('Le membre a bien été ajouté');
        }
        this.closed.emit(true);
      },
      (err) => {
        this.ownerAlreadyLinked = true;
      }
    );
  }
}
