import { Injectable } from '@angular/core';
import { ActivatedRouteSnapshot, Resolve, Router } from '@angular/router';
import { forkJoin, Observable, Subject } from 'rxjs';
import { catchError, map } from 'rxjs/operators';
import { Structure } from '../models/structure.model';
import { StructureService } from '../services/structure.service';

import { SearchService } from '../structure-list/services/search.service';
import { Utils } from '../utils/utils';

@Injectable()
export class StructureResolver implements Resolve<Structure> {
  public subject: Subject<Structure> = new Subject();

  constructor(
    private structureService: StructureService,
    private searchService: SearchService,
    private router: Router,
    public utils: Utils
  ) {}

  resolve(route: ActivatedRouteSnapshot): Observable<Structure> {
    const structureId = route.params.id;

    if (structureId) {
      forkJoin({
        structure: this.structureService.getStructure(structureId).pipe(
          map((res) => res),
          catchError(() => {
            this.router.navigate(['/home']);
            return new Observable<Structure>();
          })
        ),
        categories: this.searchService.getCategories(),
      }).subscribe((res) => {
        const structure = this.utils.setServiceCategories(res.categories, res.structure) as Structure;
        // return res;
        this.setSubject(structure);
      });

      return this.getSubject();
    }
  }

  setSubject(structure: Structure): void {
    this.subject.next(structure);
  }

  getSubject(): Observable<Structure> {
    return this.subject.asObservable();
  }
}
