import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { Category } from '../../../structure-list/models/category.model';
import { Module } from '../../../structure-list/models/module.model';
import { ButtonType } from '../button/buttonType.enum';

@Component({
  selector: 'app-accompaniment-picker',
  templateUrl: './accompaniment-picker.component.html',
  styleUrls: ['./accompaniment-picker.component.scss'],
})
export class AccompanimentPickerComponent implements OnInit, OnChanges {
  @Input() personalOfferForm: UntypedFormGroup;
  @Input() onlineProcedures: Category;
  @Output() validateForm = new EventEmitter<any>();

  public buttonTypeEnum = ButtonType;
  public selectedModules: Module[] = [];

  ngOnInit(): void {
    this.initSelectedModule();
  }
  ngOnChanges(): void {
    this.initSelectedModule();
  }

  private initSelectedModule() {
    this.selectedModules = this.onlineProcedures.modules.filter((module) =>
      this.personalOfferForm.get('categories').get('onlineProcedures').value.includes(module.id)
    );
    this.validateForm.emit();
  }

  public toogleResult(module: Module): void {
    if (this.isSelectedModule(module)) {
      const index = this.selectedModules.findIndex((_module) => _module.id === module.id);
      this.selectedModules.splice(index, 1);
    } else {
      this.selectedModules.push(module);
    }
    this.personalOfferForm
      .get('categories')
      .get('onlineProcedures')
      .patchValue(this.selectedModules.map((module) => module.id));
    this.personalOfferForm.get('categories').get('onlineProcedures').markAsDirty();
  }

  public isSelectedModule(module: Module): boolean {
    if (this.selectedModules && this.selectedModules.includes(module)) return true;
    return false;
  }
}
