import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from '@angular/core';
import { cloneDeep, remove } from 'lodash';
import { CategoriesToggle } from 'src/app/models/categoriesToggle.model';
import { Category } from '../../../structure-list/models/category.model';
import { Module } from '../../../structure-list/models/module.model';
import { CategoryEnum } from '../../enum/category.enum';
import { ButtonType } from '../button/buttonType.enum';

@Component({
  selector: 'app-training-type-picker',
  templateUrl: './training-type-picker.component.html',
  styleUrls: ['./training-type-picker.component.scss'],
})
export class TrainingTypePickerComponent implements OnInit, OnChanges {
  @Input() public baseSkills: string[];
  @Input() public advancedSkills: string[];
  @Input() public trainingCategories: CategoriesToggle[];
  @Output() selectedType: EventEmitter<Category[]> = new EventEmitter<Category[]>();

  public buttonTypeEnum = ButtonType;
  public categories: Category[] = [];
  public categoriesExpanded: string[] = [];
  public selectedChoices: Category[] = [];

  ngOnInit(): void {
    this.trainingCategories.forEach((data) => {
      this.categories.push(data.category);
    });
    this.initSelectedChoice();
    this.selectedChoices.forEach((category) => {
      if (category.modules.length) {
        this.categoriesExpanded.push(category.id);
      }
    });
  }

  ngOnChanges(): void {
    this.initSelectedChoice();
  }

  private initSelectedChoice() {
    this.selectedChoices = cloneDeep(this.categories);
    this.selectedChoices.forEach((category) => {
      let selectedModulesId: string[] = [];
      switch (category.id) {
        case CategoryEnum.baseSkills:
          selectedModulesId = this.baseSkills;
          break;
        case CategoryEnum.advancedSkills:
          selectedModulesId = this.advancedSkills;
          break;
        default:
          throw new Error(`Unimplemented training type ${category.id}`);
      }
      category.modules = category.modules.filter((module) => selectedModulesId.includes(module.id));
    });
  }

  public isCategorieExpanded(id: string): boolean {
    return this.categoriesExpanded.includes(id);
  }

  public toggleCollapse(id: string): void {
    if (this.isCategorieExpanded(id)) {
      const index = this.categoriesExpanded.findIndex((categorieId) => categorieId === id);
      this.categoriesExpanded.splice(index, 1);
    } else {
      this.categoriesExpanded.push(id);
    }
  }

  public isModulePicked(categorie: Category, module: Module) {
    const index = this.selectedChoices.findIndex((_categorie) => _categorie.id === categorie.id);
    if (index === -1) return false;
    return this.selectedChoices[index].modules.findIndex((_module) => _module.id === module.id) > -1;
  }

  public pickChoice(categorie: Category, module: Module): void {
    const index = this.selectedChoices.findIndex((_categorie) => _categorie.id === categorie.id);
    if (this.selectedChoices[index].modules.some((choices) => choices.id === module.id)) {
      remove(this.selectedChoices[index].modules, module);
    } else {
      this.selectedChoices[index].modules.push(module);
    }
    this.selectedType.emit(this.selectedChoices);
  }
  public pickAllCategory(categorie: Category): void {
    const index = this.selectedChoices.findIndex((_categorie) => _categorie.id === categorie.id);
    if (this.getCategoryCheckboxStatus(categorie) === 'checked') {
      this.selectedChoices[index].modules = [];
    } else {
      for (let module of categorie.modules) {
        if (!this.isModulePicked(categorie, module)) this.selectedChoices[index].modules.push(module);
      }
    }
    this.selectedType.emit(this.selectedChoices);
  }

  public getCategoryCheckboxStatus(c: Category): string {
    const index = this.selectedChoices.findIndex((_categorie) => _categorie.id === c.id);
    if (this.selectedChoices[index].modules.length === c.modules.length) {
      return 'checked';
    } else if (this.selectedChoices[index].modules.length === 0) {
      return 'unchecked';
    } else if (this.selectedChoices[index].modules.length && this.selectedChoices[index].modules.length > 0) {
      return 'halfChecked';
    }
  }
}
