import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GeoJson } from '../map/models/geojson.model';
import { Structure } from '../models/structure.model';
import { AuthService } from '../services/auth.service';
import { StructureService } from '../services/structure.service';
import { ButtonType } from '../shared/components/button/buttonType.enum';
import { PrintService } from '../shared/service/print.service';

@Component({
  selector: 'app-structure-list',
  templateUrl: './structure-list.component.html',
  styleUrls: ['./structure-list.component.scss'],
})
export class StructureListComponent implements OnChanges, OnInit {
  @Input() public isOrientation: boolean = false;
  @Input() public structureList: Structure[];
  @Input() public location: GeoJson;
  @Input() public selectedStructure: Structure = new Structure();
  @Input() public selectedStructureList: Structure[] = [];
  @Output() public displayMapMarkerId: EventEmitter<string> = new EventEmitter<string>();
  @Output() public selectedMarkerId: EventEmitter<string> = new EventEmitter<string>();
  @Output() public updatedStructure: EventEmitter<Structure> = new EventEmitter<Structure>();
  @Output() public structureSelected: EventEmitter<Structure[]> = new EventEmitter<Structure[]>();

  public buttonTypeEnum = ButtonType;
  public structure: Structure;
  public printMode = false;
  public isLoading = true;

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private structureService: StructureService,
    private authService: AuthService,
    private printService: PrintService
  ) {
    this.route.queryParams.subscribe((queryParams) => {
      if (queryParams.id) {
        if (!this.structure) {
          this.structureService.getStructure(queryParams.id).subscribe((s) => {
            this.showDetails(new Structure(s));
          });
        }
      } else {
        this.closeDetails();
      }
    });
  }

  ngOnInit(): void {
    if (this.printMode) {
      this.printService.onDataReady();
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.selectedStructure && this.selectedStructure) {
      this.showDetails(this.selectedStructure);
      this.router.navigate([], {
        relativeTo: this.route,
        queryParams: {
          id: this.selectedStructure._id,
        },
      });
    }
    if (changes.structureList) {
      document.getElementById('listCard').scrollTo(0, 0);
      this.isLoading = changes.structureList.firstChange;
    }
  }

  public addToList(structure: Structure): void {
    let index = this.selectedStructureList.findIndex((elem) => elem._id == structure._id);
    if (index > -1) {
      this.selectedStructureList = this.selectedStructureList.filter((elem) => {
        return elem._id != structure._id;
      });
    } else {
      this.selectedStructureList.push(structure);
    }
    this.selectedStructureList = this.selectedStructureList.slice();
    this.structureSelected.emit(this.selectedStructureList);
  }

  public isInPrintList(id: string): boolean {
    return this.selectedStructureList.findIndex((elem) => elem._id == id) > -1 ? true : false;
  }

  public addStructure(): void {
    if (!this.authService.isLoggedIn()) {
      this.router.navigateByUrl('/login');
    } else {
      this.router.navigateByUrl('/form/structure');
    }
  }

  public showDetails(event: Structure): void {
    this.structure = event;
    this.selectedMarkerId.emit(this.structure._id);
  }

  public closeDetails(): void {
    this.selectedMarkerId.emit();
  }

  public handleCardHover(structure: Structure): void {
    this.displayMapMarkerId.emit(structure._id);
  }

  public mouseLeave(): void {
    this.displayMapMarkerId.emit(undefined);
  }

  public print() {
    this.printService.printDocuments('structures-search', this.structureList);
  }
}
